# -*- coding: utf-8 -*-
# Generated by Django 1.9.3 on 2016-05-01 18:49
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('qna', '0011_question_has_answer'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='question',
            options={'ordering': ['-created_at'], 'verbose_name': 'Вопрос', 'verbose_name_plural': 'Вопросы'},
        ),
        migrations.AlterField(
            model_name='question',
            name='last_update',
            field=models.DateTimeField(auto_now=True, null=True),
        ),
    ]



from django import forms
from django.contrib.auth.models import User
from django.contrib.contenttypes.fields import GenericForeignKey
from django.utils.translation import ugettext as _

from .models import Question, Answer
from gfk_models.gfk_helpers.functions.text_processors import TextProcessors


class QuestionForm(forms.ModelForm):
    text_allowed_tags = ['img', 'div', 'p', 'h2', 'h3', 'h4']
    text_allowed_attrs = ['alt', 'rel', 'href']

    class Meta:
        model = Question
        fields = ['title', 'text', 'target_content_type', 'target_id', 'author']

    def clean_text(self):
        text = self.cleaned_data['text']
        text = TextProcessors.text_sanitize(text)
        text = TextProcessors.text_linkify(text)
        return text

    def clean(self):
        clean = super(QuestionForm, self).clean()
        # print('clean', clean)
        return clean

    def is_valid(self):
        is_valid = super(QuestionForm, self).is_valid()
        # print('is_valid', is_valid)
        return is_valid

    def save(self, commit=True):
        return super(QuestionForm, self).save(commit)


class AnswerForm(forms.ModelForm):
    class Meta:
        model = Answer
        fields = ['text', 'author', 'question']

    def clean_text(self):
        text = self.cleaned_data['text']
        text = TextProcessors.text_sanitize(text)
        text = TextProcessors.text_linkify(text)
        return text


from django.db import models
from gfk_models.vote.models import VoteAbstract, VoteStatsAbstract


class Like(VoteAbstract):
    class Meta:
        unique_together = ['target_content_type', 'target_id', 'author']


class LikeStats(VoteStatsAbstract):
    model = Like
    positive = models.IntegerField(default=0)
    negative = models.IntegerField(default=0)

    recount_attr_list = ['count', 'sum', 'positive', 'negative']

    def recount_positive(self):
        return self.get_recount_queryset().filter(rating=+1).count()

    def recount_negative(self):
        return self.get_recount_queryset().filter(rating=-1).count()


LikeStats.setup_relation(Like)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.3 on 2016-04-24 15:42
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('qna', '0002_auto_20160424_1425'),
    ]

    operations = [
        migrations.AddField(
            model_name='question',
            name='_target',
            field=models.BooleanField(default=True),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.3 on 2016-04-26 09:08
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('qna', '0004_question_slug'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='question',
            unique_together=set([]),
        ),
    ]



from .models import ReviewStats, Review
from .forms import ReviewForm

from gfk_models.gfk_base.mixins import GFKItemsStatsManager, GFKBaseMixin
from django.contrib.contenttypes.fields import GenericRelation


class ReviewStatsManager(GFKItemsStatsManager):
    stats_model = ReviewStats

    item_model = Review
    item_form = ReviewForm

    create_or_update_fields = ['author']


class ReviewsMixin(GFKBaseMixin):
    reviews = GenericRelation(Review,
                              related_query_name='target',
                              object_id_field='target_id',
                              content_type_field='target_content_type')

    def reviews_manager(self):
        return ReviewStatsManager(self)

    class Meta:
        abstract = True


from django.shortcuts import render
from django.http import JsonResponse
from django.contrib.contenttypes.models import ContentType
from django.contrib.auth.decorators import login_required
from django.views.decorators.csrf import csrf_exempt
from django.http.response import JsonResponse
from django.utils.decorators import method_decorator
from django.forms.forms import ValidationError

from gfk_models.gfk_base.views import GFKCreateView
from .forms import ReviewForm
from .models import Review
from gfk_models.gfk_helpers.functions.common import CommonHelpers


class ReviewCreateView(GFKCreateView):
    @method_decorator(csrf_exempt)
    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        return super(ReviewCreateView, self).dispatch(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        # print('populate_target')
        # self.populate_self(request, *args, **kwargs)

        author = CommonHelpers.get_user__by_request(request)
        tmp_post = request.POST.copy()
        tmp_post.update({'author': author.id})
        request.POST = tmp_post

        try:
            self.target.reviews_manager().add(request.POST.dict())
            return JsonResponse({
                'success': True,
            })

        except ValidationError as ex:
            form = ex.args[1]
            print(form.errors)

            return JsonResponse({
                'success' : False,
                'errors' : [(k, v[0]) for k, v in form.errors.items()],
            })



# @login_required
# def add(request):
#     data_dict = request.POST.copy()
#
#     target_type = ContentType.objects.get(app_label=data_dict['target_app_label'], model=data_dict['target_model'])
#     target = target_type.model_class().objects.get(id=data_dict['target_id'])
#
#     # data_dict['target_id'] = target.id
#     # data_dict['target_content_type'] = target_type.id
#     author = CommonHelpers.get_user(request)
#     data_dict['author'] = author.id
#
#     review_list = ReviewList.objects.get(_target=target)
#     data_dict['review_list'] = review_list
#
#     try:
#         review = Review.objects.get(review_list=review_list, author=author)
#         update = True
#     except Exception as ex:
#         print(ex)
#         update = False
#         review = Review()
#
#     form = ReviewForm(data_dict, instance=review)
#
#     if form.is_valid():
#         form.save()
#         if update:
#             return JsonResponse({
#                 'success': True,
#                 'create': False
#             })
#         else:
#             return JsonResponse({
#                 'success': True,
#                 'create': True
#             })
#     else:
#         print(form.errors)
#
#     return JsonResponse({
#         'success' : False,
#         'errors' : [(k, v[0]) for k, v in form.errors.items()],
#     })





from django.contrib import admin
from .models import Question, Answer


class AnswerInline(admin.StackedInline):
    fk_name = 'question'
    model = Answer


class QuestionAdmin(admin.ModelAdmin):
    exclude = ['last_update']
    list_display = ['title', 'author','last_update']

    inlines = [AnswerInline]

    model = Question

admin.site.register(Question, QuestionAdmin)

class AnswerAdmin(admin.ModelAdmin):
    list_display = ['author', 'last_update', 'created_at']
    model = Answer

admin.site.register(Answer, AnswerAdmin)


# class AnswerAdmin(admin.ModelAdmin):
#     exclude = ['last_update']
#     list_display = ['author','last_update']
#     model = Answer
#
# admin.site.register(Answer, AnswerAdmin)



from django.contrib.auth.decorators import login_required
from django.utils.decorators import method_decorator
from django.contrib.contenttypes.models import ContentType
from django.core.urlresolvers import reverse
from django.shortcuts import redirect
from django.template.response import TemplateResponse
from django.http.response import JsonResponse
from django.forms import ValidationError
from django.contrib.auth.mixins import LoginRequiredMixin
from django.views import generic

from gfk_models.gfk_base.views import GFKListView, GFKDetailView, GFKCreateView, GFKUpdateView
from gfk_models.gfk_helpers.functions.common import CommonHelpers
from .forms import QuestionForm, AnswerForm
from .models import Question, Answer
from hitcount.views import HitCountDetailView


class QuestionCreateView(LoginRequiredMixin, GFKCreateView):
    model = Question
    form_class = QuestionForm
    template_name = "qna/add.html"

    def __init__(self, *args, **kwargs):
        # self.login_url = reverse('auth:sign_in')
        return super().__init__(*args, **kwargs)

    @method_decorator(login_required)
    def post(self, request, *args, **kwargs):

        post_dict = request.POST.copy()
        post_dict.update({'author': CommonHelpers.get_user(request).id})
        request.POST = post_dict

        return super(QuestionCreateView, self).post(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        data = super().get_context_data(**kwargs)
        data['submit_button_text'] = "Задать вопрос"
        return data


class QuestionUpdateView(LoginRequiredMixin, generic.UpdateView):
    model = Question
    form_class = QuestionForm
    pk_url_kwarg = 'q_id'

    def __init__(self, *args, **kwargs):
        # self.login_url = reverse('auth:sign_in')
        return super().__init__(*args, **kwargs)

    template_name = "qna/add.html"

    def dispatch(self, request, *args, **kwargs):

        return super().dispatch(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):

        post_dict = request.POST.copy()
        post_dict.update({'author': CommonHelpers.get_user(request).id})
        request.POST = post_dict

        return super(QuestionUpdateView, self).post(request, *args, **kwargs)

    def get_object(self, queryset=None):
        object = super().get_object(queryset)

        if object.author.id != self.request.user.id:
            raise ValidationError('Редактировать можно только свои вопросы и ответы.')

        return object

    def get_context_data(self, **kwargs):
        data = super().get_context_data(**kwargs)
        data['submit_button_text'] = "Сохранить вопрос"
        return data


class QuestionListView(GFKListView):
    model = Question
    paginate_by = 10
    paginate_orphans = 5

    template_name = "qna/list.html"

    def get_context_data(self, **kwargs):
        data = super().get_context_data(**kwargs)
        data['sidebar_popular'] = Question.get_popular_today()

        return data


class QuestionDetailView(HitCountDetailView, GFKDetailView):
    model = Question
    count_hit = True
    pk_url_kwarg = 'q_id'

    template_name = "qna/single.html"

    def get_target(self):
        return

    def get_context_data(self, **kwargs):
        data = super().get_context_data(**kwargs)
        data['target'] = data['object'].get_target()
        data['sidebar_popular'] = Question.get_popular_today()

        return data

    def get(self, request, *args, **kwargs):
        question = Question.objects.get(id=kwargs['q_id'])

        if question.get_slug() != kwargs['q_slug']:
            return redirect(question.get_absolute_url())
        else:
            return super().get(request, *args, **kwargs)


class QuestionAcceptAnswerView(LoginRequiredMixin, generic.DetailView):
    model = Question
    pk_url_kwarg = 'q_id'

    def __init__(self, *args, **kwargs):
        # self.login_url = reverse('auth:sign_in')
        return super().__init__(*args, **kwargs)

    @method_decorator(login_required)
    def get(self, request, *args, **kwargs):
        question = Question.objects.get(id=kwargs['q_id'])
        result = question.toggle_accepted_answer(kwargs['a_id'], request.user)

        return JsonResponse({
            'checked': result
        })


class AnswerCreateView(LoginRequiredMixin, generic.CreateView):
    model = Answer
    form_class = AnswerForm

    template_name = "qna/answer_add.html"

    def __init__(self, *args, **kwargs):
        # self.login_url = reverse('auth:sign_in')
        return super().__init__(*args, **kwargs)

    def post(self, request, *args, **kwargs):
        post_dict = request.POST.copy()
        post_dict.update({'author': CommonHelpers.get_user(request).id})
        request.POST = post_dict

        return super().post(request, *args, **kwargs)


class AnswerUpdateView(LoginRequiredMixin, generic.UpdateView):
    model = Answer
    pk_url_kwarg = 'a_id'
    form_class = AnswerForm

    template_name = "qna/answer_add.html"

    def __init__(self, *args, **kwargs):
        # self.login_url = reverse('auth:sign_in')
        return super().__init__(*args, **kwargs)

    def post(self, request, *args, **kwargs):
        post_dict = request.POST.copy()
        post_dict.update({'author': CommonHelpers.get_user(request).id})
        request.POST = post_dict

        return super().post(request, *args, **kwargs)

    def get_object(self, queryset=None):
        object = super().get_object(queryset)

        if object.author.id != self.request.user.id:
            raise ValidationError('Редактировать можно только свои вопросы и ответы.')

        return object



from django.contrib.auth.models import User
from django.core.exceptions import ValidationError
from django.db import models
from django.db.models import Avg, Sum
from django.utils.translation import ugettext_lazy as _

from gfk_models.gfk_base.models import GFKModel, GFKStats


class VoteStatsAbstract(GFKStats):
    # total sum for connected vote objects
    sum = models.IntegerField(default=0)
    # Model to count stats on
    model = None

    recount_attr_list = ['count', 'sum']

    class Meta:
        abstract = True

    def recount_sum(self):
        return self.get_recount_queryset().aggregate(Sum('rating'))['rating__sum']


class VoteAbstract(GFKModel):
    # vote_list = models.ForeignKey(MyVoteList)
    # * required
    # vote_list_attr_name = 'vote_list'

    ACCEPTED_VALUES = [-1, +1]

    # Vote author
    author = models.ForeignKey(User)

    # Current rating
    rating = models.IntegerField()

    last_update = models.DateField(auto_now=True, null=True)

    def clean_rating(self):
        if self.rating not in self.ACCEPTED_VALUES:
            raise ValidationError(_('Rating field accept only this values: %s') % self.ACCEPTED_VALUES)

    class Meta:
        abstract = True



# -*- coding: utf-8 -*-
# Generated by Django 1.9.3 on 2016-04-25 10:11
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('review', '0005_review_last_update'),
    ]

    operations = [
        migrations.CreateModel(
            name='ReviewList',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('target_id', models.PositiveIntegerField()),
                ('_target', models.BooleanField(default=True)),
                ('count', models.IntegerField(default=0)),
                ('sum', models.IntegerField(default=0)),
                ('avg', models.FloatField(default=0)),
                ('target_content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AlterModelOptions(
            name='review',
            options={},
        ),
        migrations.AlterUniqueTogether(
            name='review',
            unique_together=set([]),
        ),
        migrations.AddField(
            model_name='review',
            name='review_list',
            field=models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE, to='review.ReviewList'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.3 on 2016-04-27 11:05
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('review', '0007_auto_20160427_0959'),
    ]

    operations = [
        migrations.AlterField(
            model_name='review',
            name='_id',
            field=models.CharField(blank=True, max_length=24, null=True),
        ),
        migrations.AlterUniqueTogether(
            name='reviewlist',
            unique_together=set([('target_content_type', 'target_id')]),
        ),
    ]



from django.conf.urls import url, include
from . import views, models


urlpatterns = [
    # url(r'^list', views.list, name='list'),
    url(r'^questions', views.QuestionListView.as_view(), name='list'),

    url(r'^qna/(?P<q_id>[\d]+)-(?P<q_slug>[\w\-]+)$',
        views.QuestionDetailView.as_view(), name='single'),
    #
    # url(r'^qna/(?P<q_id>[\d]+)-(?P<q_slug>[\w\-]+)/(?P<answer_id>[\d]+)$',
    #     views.QuestionAcceptAnswerView.as_view(), name='accept_answer'),
    # like dislike urls for question
    # url(r'^qna/(?P<q_id>[\d]+)-(?P<q_slug>[\w\-]+)/',
    #     include('gfk.like.urls'),
    #     {
    #         'target_model': models.Question,
    #         'target_request_attr': 'q_id',
    #         'target_retrieve_attr': 'id'
    #     }),
    url(r'^qna/add', views.QuestionCreateView.as_view(), name='add'),
]


from django.db import models
from gfk_models.gfk_base.models import GFKModel
from sorl.thumbnail import ImageField


class Screenshot(GFKModel):
    src = ImageField(upload_to="screenshots/", height_field="height", width_field="width")
    width = models.IntegerField(null=True, blank=True)
    height = models.IntegerField(null=True, blank=True)
    description = models.TextField(null=True, blank=True)

    class Meta:
        verbose_name = 'Скриншот'
        verbose_name_plural = 'Скриншоты'

    def __str__(self):
        return str(self.src)
